function [XLt, XRt, flag, iter, re, time] = Attr_Main_Block_ugly(A, B, node_label1, node_label2, l_node, tol, k1, max_iter, alpha)
%% function of block coordinate method for attributed Sylvester equation
% #-----------------------------------------------------------------------------------------------------------#
% -- INPUTS:
%   A, B: adjacency matrix of two graphs
%   node_label1/node_label2: node label vectors
%   l_node: # of node labels
%   tol: tolerance of the solver, usually a small scalar like 1e-6
%   k1: subspace size of A, Aq respectively, usually a small scalar in [2,60]
%   max_iter: max iteration number
%   alpha: parameter of the Sylvester equation; in (0,1)
% -- OUTPUTS:
%   XLt, XRt: the low-rank representation of solution
%   flag: 1: converge 0: not converge
%   iter: iteration number
%   re: residual value
%   time: running time
% #------------------------------------------------------------------------------------------------------------#
tic;
m = size(A, 1); n = size(B, 1);
Da = sum(A, 2).^(-0.5);
Db = sum(B, 2).^(-0.5);
Da(Da==Inf) = 0;
Db(Db==Inf) = 0;
Da = sparse(1:m,1:m,Da);
Db = sparse(1:n,1:n,Db);
Acel = cell(l_node, l_node);
Bcel = cell(l_node, l_node);
XLt = cell(1,l_node);
XRt = cell(1,l_node);
BLt = cell(1,l_node);
BRt = cell(1,l_node);
l = @Moperator_Block_v2;
for i=1:l_node
    [rn1,~,~] = find(node_label1 == i);
    [rn2,~,~] = find(node_label2 == i);        
    N1i = sparse(rn1, rn1, 1, m, m);
    N2i = sparse(rn2, rn2, 1, n, n);
    for j = 1:l_node
        
        [rn3,~,~] = find(node_label1 == j);
        [rn4,~,~] = find(node_label2 == j);
        
        N1j = sparse(rn3, rn3, 1, m, m);
        N2j = sparse(rn4, rn4, 1, n, n);
        
        if i == j
            BLt{j} = N2j * ones(n,1)/((m*n)^0.5); 
            BRt{j} = ones(1,m)/((m*n)^0.5)*N1j; 
            XLt{j} = sparse(n,1);
            XRt{j} = sparse(1,m);
        end
        
        
        An = N1i * A * N1j; 
        Bn = N2i * B * N2j; 
        
        Aj = alpha^(0.5)*Da * An * Da; 
        Bj = alpha^(0.5)*Db * Bn * Db; 
        
        Acel{i, j} = Aj;
        Bcel{i, j} = Bj;
    end
end

re = 0;
Re0L = ones(n, 1)/(m*n);
Re0R = ones(1, m);
% re = norm(Re0L*Re0R, 'fro');
for i = 1 : size(Re0L, 1)
    for j = 1:size(Re0L, 2)
        re = re + Re0L(i, :)*Re0R(:, j);
    end
end

iter = 0;flag = 0;
while iter < max_iter
    iter = iter +1;
    
    fprintf('iteration %d ',iter);
    if re < tol
%         XBL = Re0L;
%         XBR = Re0R;
        flag = 1;
        break;
    else
        for i = 1:l_node
                [Ol, Or] = Operator2_Block_v2(Acel, Bcel, XLt, XRt, l_node, i);
                BL = [BLt{i}, Ol];
                BR = [BRt{i}; Or];
            for j = 1:l_node                
                if i == j 
                    % call main4Block
                    if toc > 3000
                        break;
                    end
                    [Xs1, Xt1, ~,it1,tim] = main4Block_v2(Acel{j, j}, Bcel{j, j}, BL, BR, 1e-7, k1, k1, 1e-8, 10, 100);
                    XLt{i} = -Xs1;
                    XRt{i} = Xt1;
                    fprintf('iter1: %d ',it1);
                    fprintf('time:%d ',tim);
                end
%                 if i == j && i ~= 1
% %                     % call main4Block
%                     [Xs1, Xt1, ~, it2, ~] = main4Block_v2(Acel{j, j}, Bcel{j, j}, BL, BR, tol, k2, k2, 1e-10, 30, 100);
%                     XLt{i} = -Xs1;
%                     XRt{i} = Xt1;
%                     fprintf('iter2: %d\n ',it2);
%                 end
            end
        end

        % THIS IS SLOW! CHANGE!
        re =0;
        for t = 1 : l_node
            [ll, lr] =  l(Acel, Bcel, XLt, XRt, l_node, t);
            rel = cat(2, BLt{t}, -XLt{t}, ll);
            rer = cat(1, BRt{t}, XRt{t}, lr);
%             for s = 1:size(rel,1)
%                 for q = 1:size(rer,2)
%                     re = re + rel(s, :)*rer(:, q);
%                 end
%             end
            re = re + norm(rel*rer, 'fro');
        end
        fprintf('residual: %d\n', re);
    end
end
time = toc;
end