function [S1, T1, flag, iter, time] = main_v2(A, Aq, L, L1, tol, m, n, epsilon, max_iter, CGmax_iter, alpha)
%% main function of Sylvester Equation method
% #-----------------------------------------------------------------------------------------------------------#
% -- INPUTS:
%   A, Aq: adjacency matrix of two graphs
%   L, L1: the low-rank representation of preference matrix
%   tol: tolerance of the solver, usually a small scalar like 1e-6
%   m, n: subspace size of A, Aq respectively, usually a small scalar in [2,60]
%   epsilon: the tolerance of the Glocal CG method used in the solver,
%   usually smaller than tol (e.g. 1e-7)
%   max_iter: max iteration number
%   CGmax_iter: max iteration number for Global CG method
%   alpha: parameter of the Sylvester equation; in (0,1)
% -- OUTPUTS:
%   S1, T1: the low-rank representation of solution
%   flag: 1: converge 0: not converge
%   iter: iteration number
%   time: running time
% #------------------------------------------------------------------------------------------------------------#
tic;
da = sum(A,2);
db = sum(Aq, 2);
n1 = size(A, 1);
n2 = size(Aq, 1);
% A = alpha^(0.5).*bsxfun(@times, bsxfun(@times, da.^(-0.5), A),
% (da.^(-0.5))');% not efficient
% Aq = alpha^(0.5).*bsxfun(@times, bsxfun(@times, db.^(-0.5), Aq),
% (db.^(-0.5))'); % not efficient
D1 = sparse(1:n1, 1:n1, da.^(-0.5));
D2 = sparse(1:n2, 1:n2, db.^(-0.5));
A(isnan(A))=0;
Aq(isnan(Aq))=0;
A = alpha^(0.5).*D1*A*D1;
Aq = alpha^(0.5).*D2*Aq*D2;


% L = R(:,1);
% L1 = ones(1, size(R,2));

iter = 0;flag = 0;U0 = L; Vv0 = L1;
while iter < max_iter
    iter = iter +1;
    TempL = Vv0'*(U0'*U0);
    TempR = Vv0;
    Tr = 0;
    for i = 1:size(Vv0, 2)
        Tr = Tr + TempL(i, :)*TempR(:, i);
    end
    if Tr^(0.5) < tol
%    if norm(U0*Vv0, 'fro') < tol
%         X = X0;
        S1 = U0;
        T1 = Vv0;
%         re = norm(U*Vv,'fro');
        flag = 1;
        break;
    else
        start = tic;
%         if iter == 1
%             f = L; g = L1;
%         else
%             [f,g] = subspace_v2(U0, Vv0);
%         end
        [f,g] = subspace_v2(U0, Vv0);
        [V,Ha] = arnoldi(A, g, n);
        [W,Hb] = arnoldi(Aq, f, m);
        Arnotime = toc(start);
        fprintf('Arnotime:%d ',Arnotime);
        
        CGstart = tic;
        E = (Hb'*W'*U0)*(Vv0*(V*Ha)) - (W(:,1:m)'*U0)*(Vv0*V(:,1:n));
        [Y, flag, CGiter, ~] =  GLCG(sparse(zeros(m,n)), E, epsilon, CGmax_iter, Ha, Hb, m, n);
        CGtime = toc(CGstart);
        fprintf('CGtime:%d ',CGtime);
        fprintf('CGiter:%d ',CGiter);
        
%         X1 = X0 + W(:, 1:m)*Y*V(:, 1:n)';
%         Re1 = Re0 - W*Hb*Y*Ha'*V' + W(:,1:m)*Y*V(:,1:n)';
        L2 = -W*Hb*Y*Ha'; P2 = V'; L3 = W(:,1:m)*Y; P3 = V(:,1:n)';
        U0 = cat(2, cat(2, U0, L2), L3);
        Vv0 = cat(2, cat(2, Vv0', P2'), P3')';
%         re = norm(U0*Vv0, 'fro');
        if iter == 1
            S0 = W(:, 1:m)*Y;
            T0 = V(:, 1:n)';
        else
            S0 = cat(2, S0, W(:, 1:m)*Y);
            T0 = cat(2, T0, V(:, 1:n)');
        end
        
%         X0 = X1;
%         Re0 = Re1;
        fprintf('out_iter:%d\n ', iter);
%         fprintf('residual:%d\n ',re);
    end
end
% X = X0;
S1 = S0;T1 = T0;
time = toc;
end
